﻿using System;

using SourceCode.Hosting.Client.BaseAPI;
using SourceCode.Workflow.Management.Criteria;
using SourceCode.Workflow.Management;

namespace K2.OnProcessErrorEvent.CustomAction
{
    public class CustomActionCode
    {
        /// <summary>
        /// Simple method that uses the SCConnectionStringBuilder to create Windows Integrated authentication.
        /// </summary>
        /// <param name="hostname">hostname of K2. Defaults to localhost</param>
        /// <param name="port">port to use in the connection string. Defaults to 5555 for the management APIs</param>
        /// <returns></returns>
        private static string GetConnectionString(string hostname = "localhost", uint port = 5555)
        {
            SCConnectionStringBuilder conBuilder = new SCConnectionStringBuilder();
            conBuilder.Port = port; 
            conBuilder.Host = hostname;
            conBuilder.Integrated = true;
            conBuilder.Authenticate = true;
            conBuilder.IsPrimaryLogin = true;
            return conBuilder.ToString();
        }



        /// <summary>
        /// Handle the process in error with some custom code
        /// </summary>
        /// <param name="processInstanceId">The process instance id</param>
        public static void HandleProcessError(int processInstanceId)
        {
            Console.WriteLine($"[HandleProcessError]: Started with parameter: {processInstanceId}");

            WorkflowManagementServer wfmApi = new WorkflowManagementServer();
            using (wfmApi.CreateConnection())
            {
                wfmApi.Connection.Open(GetConnectionString());

                // We're using the WorkflowManagement API to retrieve some details about the specific process instance.
                ProcessInstanceCriteriaFilter filter = new ProcessInstanceCriteriaFilter();
                filter.AddRegularFilter(ProcessInstanceFields.ProcInstID, Comparison.Equals, processInstanceId);
                ProcessInstances procInstances = wfmApi.GetProcessInstancesAll(filter);
                
                // Check to make sure there's only one error event
                if (procInstances.Count != 1)
                {
                    throw new Exception($"[HandleProcessError] Did not find exactly 1 process instance (found {procInstances.Count}). Aborting.");
                }

                // retrieve the error message for the specific process and write some output about it.
                ProcessInstance procInst = procInstances[0];
                string procError = wfmApi.GetError(procInst.ProcID, processInstanceId);

                Console.WriteLine($"[HandleProcessError]: ProcessFullname: {procInst.ProcSetFullName}");
                Console.WriteLine($"[HandleProcessError]: ProcID: {procInst.ProcID}");
                Console.WriteLine($"[HandleProcessError]: ExecutingProcID: {procInst.ExecutingProcID}");
                Console.WriteLine($"[HandleProcessError]: ProcessInstanceID: {procInst.ID}");
                Console.WriteLine($"[HandleProcessError]: ProcessFolio: {procInst.Folio}");
                Console.WriteLine($"[HandleProcessError]: Error: {procError}");
                

                // Interact with process if this is needed. Change Workflows\\MyWorkflow to your actual workflow name
                if (string.Compare(procInst.ProcSetFullName, "Workflows\\My Workflow") == 0)
                {
                    Console.WriteLine($"[HandleProcessError]: Restarting process {procInst.ID} using GotoActivity(Start) as it matches the process name.");
                    wfmApi.GotoActivity(processInstanceId, "Start");
                }

                wfmApi.Connection.Close();
            }
        }
    }
}
