﻿using ClientAPI = SourceCode.EventBus.ClientAPI;
using AuthoringAPI = SourceCode.EventBus.Authoring;
using SourceCode.Hosting.Client.BaseAPI;
using System;
using System.Xml;
using System.Reflection;

namespace K2.OnProcessErrorEvent.Installer
{
    /// <summary>
    /// Simple program to install/deploy a custom event with a simple mapping.
    /// Please note the using statements at the top of this file.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Simple method that uses teh SCConnectionStringBuilder to create a Windows Integrated authentication.
        /// </summary>
        /// <param name="hostname">hostname of K2. Defaults to localhost</param>
        /// <param name="port">port to use in the connection string. Defaults to 5555 for the management APIs</param>
        /// <returns></returns>
        public static string GetConnectionString(string hostname = "localhost", uint port = 5555)
        {
            SCConnectionStringBuilder conBuilder = new SCConnectionStringBuilder();
            conBuilder.Port = port;
            conBuilder.Host = hostname;
            conBuilder.Integrated = true;
            conBuilder.Authenticate = true;
            conBuilder.IsPrimaryLogin = true;
            return conBuilder.ToString();
        }

        


        private static int GetCustomEventByName(string name)
        {
            ClientAPI.Client eventClient = new ClientAPI.Client();
            int retId = 0;
            using (eventClient.CreateConnection())
            {
                eventClient.Connection.Open(GetConnectionString());

                string customEvent = eventClient.GetCustomEventsPerFilter("all"); // you only get your own.
                XmlDocument xml = new XmlDocument();
                xml.LoadXml(customEvent);

                foreach (XmlNode node in xml.ChildNodes[1].ChildNodes)
                {
                    if (node.Attributes["name"].InnerText == name)
                    {
                        retId = int.Parse(node.Attributes["id"].InnerText);
                        break;
                    }
                }



                eventClient.Connection.Close();
            }
            return retId;

        }

        private static MethodInfo GetMethod(Type t)
        {

            MethodInfo[] methods = t.GetMethods(BindingFlags.Public | BindingFlags.Static);

            if (methods.Length == 0)
            {
                throw new Exception($"Failed to find any public and static methods in class '{t}'.");
            }
            if (methods.Length > 1)
            {
                Console.WriteLine("Found multiple public and static methods in assembly. Listing items:");
                foreach (MethodInfo meth in methods)
                {
                    Console.WriteLine($"- {meth.ToString()}");
                }

            }
            MethodInfo method = methods[0];
            Console.WriteLine($"Found {methods.Length} methods. Using {method.ToString()}");
            return method;
        }


        private static Type GetType(string pathToAssembly)
        {
            Assembly asm = Assembly.LoadFile(pathToAssembly);

            Type[] types = asm.GetTypes();
            if (types.Length == 0)
            {
                throw new Exception($"Failed to find any types in {pathToAssembly}.");
            }

            Type t = types[0];
            return t;
        }


        static void Main(string[] args)
        {

            const string eventName = "OnProcessErrorCustomEvent";
            const string eventDescription = "Custom event to handle process going into error state.";
            // TODO The path of the assembly to use for the custom code.
            // TODO We need to add this path to the K2hostserver.exe.config file under 'assemblies'
            const string assemblyPath = @"C:\Dev\K2.OnProcessErrorEvent\K2.OnProcessErrorEvent.CustomAction\bin\Debug\K2.OnProcessErrorEvent.CustomAction.dll";
            
            int eventId = GetCustomEventByName(eventName);
            if (eventId == 0)
            {
                Console.WriteLine($"No custom event with name '{eventName}' found. Creating new custom event...");

            }

            ClientAPI.EventAuthoring eventClient = new ClientAPI.EventAuthoring();
            using (eventClient.CreateConnection())
            {
                eventClient.Connection.Open(GetConnectionString());


                AuthoringAPI.CustomEvent customEvent = new AuthoringAPI.CustomEvent(
                    eventId, // Pass in the existing ID to update the existing event. Pass in 0 if we create a new event.
                    eventName,
                    eventDescription,
                    "OnProcessError",
                    "OnProcessError",
                    (int)SourceCode.EventBus.Common.Enumerators.EventType.Custom_Notification);
                customEvent.SystemID = "bc76ec99-6acd-4e9a-8dd8-db05b6be30ee"; // Constructor doesn't allow us to provide this value. It's hardcoded within K2.

                // If we created a new event, we can now get the real ID.
                eventId = eventClient.UpdateCustomEvent(customEvent);
                Console.WriteLine($"Custom Event created/updated. EventID: {eventId}");


                // Get some details about the assembly we want to use
                Type classType = GetType(assemblyPath);
                MethodInfo methodSignature = GetMethod(classType);

                Console.WriteLine($"Found class '{classType.ToString()}' with method '{methodSignature.ToString()}'.");
                // Create a policy mapping. This is a mapping between the event and the actual thing (code) that needs to happen.
                AuthoringAPI.PolicyMapping polMapping = new AuthoringAPI.PolicyMapping(
                    eventId,
                    assemblyPath,
                    classType.ToString(),
                    methodSignature.ToString()
                    );


                // Create one mapping for the first parameter
                // We're assuming that the method signature has a process Instance Id as first parameter
                AuthoringAPI.InputMapping map = new AuthoringAPI.InputMapping();
                map.PolicyField = new AuthoringAPI.PolicyField();
                map.PolicyField.Name = "processInstanceId";
                map.PolicyField.IsList = false;
                map.PolicyField.IsRequired = true;
                map.PolicyField.PolicyType = "system.int32";
                AuthoringAPI.SourceField sf = new AuthoringAPI.SourceField();
                AuthoringAPI.SourceFieldPart sfp = new AuthoringAPI.SourceFieldPart();
                sfp.DataProvider = "SourceCode.Workflow.Design.WorkflowContextFieldPart";
                sfp.DataProviderData = "ProcessId";
                sfp.DataType = "system.int32";
                sfp.DisplayName = "ProcessId";
                sfp.IsList = false;
                sfp.IsValueType = false;
                sf.SourceFieldParts.Add(sfp);
                map.SourceFields.Add(sf);
                polMapping.InputMappings.Add(map);
                Console.WriteLine("Added Process InstanceID Mapping.");

                // Map the custom event, policy and update the eventpolicy.
                customEvent.ActionPolicyMappings.Add(polMapping);
                eventClient.UpdateEventPolicyMapping(eventId, polMapping, 2); // 2 is hardcoded


                eventClient.Connection.Close();
            }
            Console.WriteLine("Registered/updated custom event.");
        }
    }
}
